@@ -1,6 +1,6 @@
 .onLoad <- function(libname, pkgname){
 
  utils::globalVariables(c(".", "trend", "s", "x", "low", "up", "endog_data",
-                          "x_lin", "d", "y_lin"))
+                          "x_lin", "d", "y_lin", "cross_id", "date_id", ":="))
 
 }
